package org.errai.samples.stockdemo.client.local;

import java.util.Random;

import org.jboss.errai.bus.client.ErraiBus;
import org.jboss.errai.bus.client.api.Message;
import org.jboss.errai.bus.client.api.MessageCallback;
import org.jboss.errai.bus.client.api.base.MessageBuilder;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;

public class StockClient implements EntryPoint {

	private static final String[] statusLightImages = new String[] {
			"unknown.png", "good.png", "bad.png", "warn.png" };
	private int statusLightImageIndex = 0;
	private String statusLightImage = "images/status/"
			+ statusLightImages[statusLightImageIndex];
	// private final Image image = new Image();

	private FlexTable managedResourcesTable = new FlexTable();
	private Random randomRow = new Random(System.currentTimeMillis());
	private Random randomColumn = new Random(System.currentTimeMillis() + 1);

	public void onModuleLoad() {
		
		setupManagedResourcesTable();
		
        ErraiBus.get().subscribe("status", new StatusReceiver());
        ErraiBus.get().subscribe("status2", new StatusReceiver());

		MessageBuilder.createMessage().toSubject("StockService")
				.command("Start").noErrorHandling()
				.sendNowWith(ErraiBus.getDispatcher());

		RootPanel.get().add(managedResourcesTable);
	}

	private void setupManagedResourcesTable() {
		for (int i = 0; i < 4; i++) {
			managedResourcesTable.insertRow(i);
			for (int j = 0; j < 2; j++) {
				managedResourcesTable.addCell(i);
				if (i == 0 || i == 2) {
					Image image = new Image();
					incrementImage();
					image.setUrl("images/status/" + statusLightImages[statusLightImageIndex]);
					managedResourcesTable.setWidget(i, j, image);
				} else {
					Label label = new Label();
					label.setText("Resource" + (i * 10 + j));
					label.setAutoHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
					label.setWordWrap(true);
					label.getElement().getStyle().setProperty("fontWeight", "bold");
					managedResourcesTable.setWidget(i, j, label);
				}
			}
		}
	}
	
	private void incrementImage()
	{
		// just some simple way of getting a different image
		++statusLightImageIndex;
		if (statusLightImageIndex == statusLightImages.length)
			statusLightImageIndex = 0;
		
		statusLightImage = "images/status/" + statusLightImages[statusLightImageIndex];
	}

	private class StatusReceiver implements MessageCallback {

        public void callback(Message message) {
            
            if ("StateChange".equals(message.getCommandType())) {
                StockClient.this.incrementImage();
                int row = randomRow.nextInt(2) * 2;
                int column = randomColumn.nextInt(2);
                Image image = (Image)managedResourcesTable.getWidget(row, column);
                image.setUrl(StockClient.this.statusLightImage);
            }
        }
    }
}
