package org.errai.samples.stockdemo.server;

import com.google.inject.Inject;

import org.jboss.errai.bus.client.api.Message;
import org.jboss.errai.bus.client.api.base.MessageBuilder;
import org.jboss.errai.bus.client.framework.MessageBus;
import org.jboss.errai.bus.client.framework.RequestDispatcher;
import org.jboss.errai.bus.server.annotations.Command;
import org.jboss.errai.bus.server.annotations.Service;

@Service
public class StockService {

	private volatile Thread statusUpdater;
	private RequestDispatcher requestDispatcher;

	@Inject
	public StockService(final RequestDispatcher dispatcher, final MessageBus bus) {
		
		this.requestDispatcher = dispatcher;
	}

	@Command("Start")
	public void start(Message message) {

        MessageBuilder
            .createConversation(message)
            .toSubject("status")
            .command("StateChange")
            .noErrorHandling()
            .reply();

        if (statusUpdater == null) {
			statusUpdater = new StatusUpdater();
			statusUpdater.setDaemon(true);
			statusUpdater.start();
		}
	}
	
	private class StatusUpdater extends Thread {
        public void run() {
            while (true) {
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                    // no op
                }
                MessageBuilder.createMessage().toSubject("status2")
                        .command("StateChange").noErrorHandling()
                        .sendGlobalWith(StockService.this.requestDispatcher);
            }
        }
	}
}
